/*
20140709
Jan Mojzis
Public domain.
*/

#include "misc.h"
#include "crypto_core_aes128encrypt.h"

static unsigned char key[crypto_core_aes128encrypt_KEYBYTES + 16];
static unsigned char out[crypto_core_aes128encrypt_OUTPUTBYTES + 16];
static unsigned char in[crypto_core_aes128encrypt_INPUTBYTES + 16];

static unsigned char o[32] = {
    0xfb, 0x69, 0xd1, 0xae, 0xce, 0x0e, 0xa0, 0x89, 
    0xfe, 0xe8, 0x07, 0xb2, 0xe1, 0x5b, 0xb3, 0xd4, 
    0x1b, 0xa9, 0x73, 0x54, 0xca, 0x8f, 0x83, 0x6c, 
    0x05, 0x51, 0x8b, 0x10, 0xe8, 0x81, 0x45, 0xc7
};

static void test_alignment(void) {

    long long i;

    for (i = 0; i < 16; ++i) {
        crypto_core_aes128encrypt(out + i, in + i, key + i, 0);
    }
}

static void test_rand(void) {

    long long i;

    for (i = 0; i < 256; ++i) {
        pseudorandombytes(key, sizeof key);
        pseudorandombytes(in, sizeof in);
        crypto_core_aes128encrypt(out, in, key, 0);
        checksum(out, crypto_core_aes128encrypt_OUTPUTBYTES);
    }
    fail_whenbadchecksum(o);
}

/* NIST testvectors from Special Publication 800-38A */
static struct vector {
    const unsigned char key[crypto_core_aes128encrypt_KEYBYTES];
    const unsigned char in[crypto_core_aes128encrypt_INPUTBYTES];
    const unsigned char out[crypto_core_aes128encrypt_OUTPUTBYTES];
} vectors[] = {
    {
        { 
            0x2b, 0x7e, 0x15, 0x16, 0x28, 0xae, 0xd2, 0xa6, 0xab, 0xf7, 0x15, 0x88, 0x09, 0xcf, 0x4f, 0x3c
        },
        {
            0xf0, 0xf1, 0xf2, 0xf3, 0xf4, 0xf5, 0xf6, 0xf7, 0xf8, 0xf9, 0xfa, 0xfb, 0xfc, 0xfd, 0xfe, 0xff
        },
        {
            0xec, 0x8c, 0xdf, 0x73, 0x98, 0x60, 0x7c, 0xb0, 0xf2, 0xd2, 0x16, 0x75, 0xea, 0x9e, 0xa1, 0xe4
        }
    },
    {
        {
            0x2b, 0x7e, 0x15, 0x16, 0x28, 0xae, 0xd2, 0xa6, 0xab, 0xf7, 0x15, 0x88, 0x09, 0xcf, 0x4f, 0x3c
        },
        {
            0xf0, 0xf1, 0xf2, 0xf3, 0xf4, 0xf5, 0xf6, 0xf7, 0xf8, 0xf9, 0xfa, 0xfb, 0xfc, 0xfd, 0xff, 0x00
        },
        {
            0x36, 0x2b, 0x7c, 0x3c, 0x67, 0x73, 0x51, 0x63, 0x18, 0xa0, 0x77, 0xd7, 0xfc, 0x50, 0x73, 0xae
        }
    },
    {
        {
            0x2b, 0x7e, 0x15, 0x16, 0x28, 0xae, 0xd2, 0xa6, 0xab, 0xf7, 0x15, 0x88, 0x09, 0xcf, 0x4f, 0x3c
        },
        {
            0xf0, 0xf1, 0xf2, 0xf3, 0xf4, 0xf5, 0xf6, 0xf7, 0xf8, 0xf9, 0xfa, 0xfb, 0xfc, 0xfd, 0xff, 0x01
        },
        {
            0x6a, 0x2c, 0xc3, 0x78, 0x78, 0x89, 0x37, 0x4f, 0xbe, 0xb4, 0xc8, 0x1b, 0x17, 0xba, 0x6c, 0x44
        }
    },
    {
        {
            0x2b, 0x7e, 0x15, 0x16, 0x28, 0xae, 0xd2, 0xa6, 0xab, 0xf7, 0x15, 0x88, 0x09, 0xcf, 0x4f, 0x3c
        },
        {
            0xf0, 0xf1, 0xf2, 0xf3, 0xf4, 0xf5, 0xf6, 0xf7, 0xf8, 0xf9, 0xfa, 0xfb, 0xfc, 0xfd, 0xff, 0x02
        },
        {
            0xe8, 0x9c, 0x39, 0x9f, 0xf0, 0xf1, 0x98, 0xc6, 0xd4, 0x0a, 0x31, 0xdb, 0x15, 0x6c, 0xab, 0xfe
        }
    },

};

static void test_vector(void) {

    long long i, j;
    unsigned char out[crypto_core_aes128encrypt_OUTPUTBYTES];

    for (i = 0; i < 4; ++i) {
        crypto_core_aes128encrypt(out, vectors[i].in, vectors[i].key, 0);
        for (j = 0; j < crypto_core_aes128encrypt_OUTPUTBYTES; ++j)
            if (out[j] != vectors[i].out[j])
                fail("crypto_core_aes128encrypt() failure");
    }
}

int main(void) {

    test_vector();
    test_alignment();
    test_rand();

    _exit(0);
}
